package org.myorg;
//using new determine-id-class
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataInput;
import java.io.DataOutput;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Date;
import java.util.List;
import java.util.StringTokenizer;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.fs.FsShell;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FSDataOutputStream;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.Writable;
import org.apache.hadoop.mapred.FileInputFormat;
import org.apache.hadoop.mapred.FileOutputFormat;
import org.apache.hadoop.mapred.JobClient;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reducer;
import org.apache.hadoop.mapred.Reporter;
import org.apache.hadoop.mapred.TextInputFormat;
import org.apache.hadoop.mapred.TextOutputFormat;
import org.apache.hadoop.util.ToolRunner;


//main class: everything (related to MapReduce) is here.
public class DiscFinder
{
  public static JobConf conf;
  public static FileSystem fs;

  //parameters from conf.txt
  static int NumPartitions;
  static String NFSPath;
  static String LocalPath;
  static String InputPath;
  static String OutputPath;
  static String TempPath;
  static int SampleRate;
  static float Epsilon;
  static long MaxPointEF;
  static int NumSlot;

  //some aux functions
  public static int floatToInt(float paramFloat)
  {
    return Integer.reverseBytes(Float.floatToIntBits(paramFloat));
  }

  public static int intToInt(int paramInt) {
    return Integer.reverseBytes(paramInt);
  }

  public static final byte[] intToByteArray(int paramInt) {
    return new byte[] { (byte)paramInt, (byte)(paramInt >>> 8), (byte)(paramInt >>> 16), (byte)(paramInt >>> 24) };
  }

  public static final byte[] floatToByteArray(float paramFloat)
  {
    int i = Float.floatToIntBits(paramFloat);
    return new byte[] { (byte)i, (byte)(i >>> 8), (byte)(i >>> 16), (byte)(i >>> 24) };
  }

  public static void main(String[] paramArrayOfString)
    throws Exception
  {
    try
    {
      BufferedReader br = new BufferedReader(new FileReader("discfinder.sh"));
      String s;
      while ((s=br.readLine())!=null)
      {
	StringTokenizer stk = new StringTokenizer(s,"=");
	if (stk.hasMoreTokens()) {
	  String pname = stk.nextToken();
	  if (stk.hasMoreTokens()) {
	    String pvalue = stk.nextToken();
	    if (pname.equals("NUMPARTITIONS"))
		    NumPartitions = Integer.parseInt(pvalue);
	    if (pname.equals("NFSPATH"))
		    NFSPath = pvalue;
	    if (pname.equals("LOCALPATH")) 
		    LocalPath = pvalue;
	    if (pname.equals("INPUTPATH"))
		    InputPath = pvalue;
	    if (pname.equals("OUTPUTPATH"))
		    OutputPath = pvalue;
	    if (pname.equals("TEMPPATH"))
		    TempPath = pvalue;
	    if (pname.equals("SAMPLERATE"))
		    SampleRate = Integer.parseInt(pvalue);
	    if (pname.equals("EPSILON"))
		    Epsilon = Float.parseFloat(pvalue);
	    if (pname.equals("MAXPOINTEF"))
		    MaxPointEF = Long.parseLong(pvalue);
	    if (pname.equals("NUMSLOT"))
		    NumSlot = Integer.parseInt(pvalue);
	  }
	}
      }
    }catch(Exception o){
      o.printStackTrace();
    }

    //then prepare for the input of the third MapReduce job (trivial)

    try
    {
      String ss[] = {"/bin/sh", "-c", "rmr -dfr trivial"};
      Process process = Runtime.getRuntime().exec(ss);
      InputStream is = process.getInputStream();
      InputStreamReader isr = new InputStreamReader(is);
      BufferedReader brr = new BufferedReader(isr);
      String s1 = null;
      while ((s1=brr.readLine())!=null);
      if (process.waitFor()!=0)
              System.out.println("exit v="+process.exitValue());

      String ss2[] = {"/bin/sh", "-c", "mkdir trivial"};
      process = Runtime.getRuntime().exec(ss2);
      is = process.getInputStream();
      isr = new InputStreamReader(is);
      brr = new BufferedReader(isr);
      while ((s1=brr.readLine())!=null);
      if (process.waitFor()!=0)
              System.out.println("exit v="+process.exitValue());

      BufferedWriter bw = new BufferedWriter(new FileWriter("./trivial/triv"));
      for (int i=0;i<NumPartitions;i++)
              bw.write(i+"\r\n");
      bw.flush();

      //upload trivial folder to HDFS
      //test it!!!!
      //also, is "hadoop dfs ..." works universally?
      //should use hdfs java api instead
      String ss3[] = {"/bin/sh", "-c", "hadoop dfs -put trivial ."};
      process = Runtime.getRuntime().exec(ss3);
      is = process.getInputStream();
      isr = new InputStreamReader(is);
      brr = new BufferedReader(isr);
      while ((s1=brr.readLine())!=null);
      if (process.waitFor()!=0)
              System.out.println("exit v="+process.exitValue());

    }catch(Exception o){}

    //prepartion done for now

    System.out.println("First stage.");

    //first map/reduce: sampling
    //naive way
    conf = new JobConf(DiscFinder.class);
    conf.setJobName("Sampling");
    conf.setMapperClass(SampleMap.class);
    conf.setCombinerClass(SampleCombine.class);
    conf.setReducerClass(SampleReduce.class);
    //124 = 32(# processing node)*4(# mapper/reducer slot per node) - 4(handle failure)
    //change when using other parameters
    conf.setNumMapTasks(NumSlot);
    conf.setNumReduceTasks(NumSlot);
    conf.setInputFormat(TextInputFormat.class);
    conf.setOutputFormat(TextOutputFormat.class);
    conf.setOutputKeyClass(Text.class);
    conf.setOutputValueClass(IntWritable.class);
    conf.setStrings("NFSPath", NFSPath);
    conf.setStrings("LocalPath", LocalPath);
    conf.setStrings("TempPath", TempPath);
    conf.setStrings("InputPath", InputPath);
    conf.setStrings("OutputPath", OutputPath);

    //0st, 1st parameters, for input directory / output directory
    //input: consisting of files with format of <id x y z> each line (text)
    //output: containing all sampled points (same format)
    FileInputFormat.setInputPaths(conf, new Path[]{ new Path(InputPath)});
    FileOutputFormat.setOutputPath(conf, new Path(TempPath));
    JobClient.runJob(conf);

    //first map/reduce done. Now processing it.
    //first copy the file from HDFS to local
    try
    {
        ToolRunner.run(conf, new FsShell(), new String[]{"-get", TempPath, "sampleoutput"});
    } catch(Exception e) {}

    //then executing the cc.class (pre-compiled) to calculate boundaries of each "cube", using sampled result
    //result of "java cc" will generate a file called "bdr"
    String ss[] = {"/bin/sh", "-c", "java cc > bdr"};
    Process process = Runtime.getRuntime().exec(ss);
    InputStream is = process.getInputStream();
    InputStreamReader isr = new InputStreamReader(is);
    BufferedReader brr = new BufferedReader(isr);
    String s1=null;
    while ((s1=brr.readLine())!=null);
    try {
      if (process.waitFor()!=0)
        System.out.println("exit v="+process.exitValue());
    } catch(Exception e){}

    //delete output file (the sampled result) in HDFS
    try
    {
      ToolRunner.run(conf, new FsShell(), new String[]{"-rmr", TempPath});
    } catch(Exception e){}

    long l = System.currentTimeMillis();

    //now prepare for the second Map/Reduce.

    //change the location here
    File localFile = new File(NFSPath+"shell");
    if (!(localFile.exists())) {
      localFile.mkdirs();
    }

    System.out.println("Second stage.");

    //second map/reduce

    conf = new JobConf(DiscFinder.class);
    conf.setJobName("DiscFinder");

    //as many as mappers (6xxx) as default
    conf.setNumReduceTasks(NumSlot);

    conf.setOutputKeyClass(IntWritable.class);
    conf.setOutputValueClass(FOFpt.class);

    conf.setMapperClass(Map.class);
    conf.setCombinerClass(Combine.class);
    conf.setReducerClass(Reduce.class);

    conf.setInputFormat(TextInputFormat.class);
    conf.setOutputFormat(TextOutputFormat.class);

    conf.setStrings("NFSPath", NFSPath);
    conf.setStrings("LocalPath", LocalPath);
    conf.setStrings("TempPath", TempPath);
    conf.setStrings("InputPath", InputPath);
    conf.setStrings("OutputPath", OutputPath);


    //same input (since it's reasonable) and output (since it's already deleted...)
    FileInputFormat.setInputPaths(conf, new Path[] { new Path(InputPath) });
    FileOutputFormat.setOutputPath(conf, new Path(TempPath));

    fs = FileSystem.get(conf);

    JobClient.runJob(conf);


    //second map/reduce done
    //now executing UnionFind
    //UnionFind will create a file called trans.trn
    String[] arrayOfString = { "/bin/sh", "-c", "java uf" };
    Process localProcess = Runtime.getRuntime().exec(arrayOfString);

    InputStream localInputStream = localProcess.getInputStream();
    InputStreamReader localInputStreamReader = new InputStreamReader(localInputStream);
    BufferedReader localBufferedReader = new BufferedReader(localInputStreamReader);
    String str2 = null;
    System.out.println("<output></output>");
    while ((str2 = localBufferedReader.readLine()) != null);
    try
    {
      if (localProcess.waitFor() != 0) {
        System.err.println("exit value = " + localProcess.exitValue());
      }

    }
    catch (Exception localException)
    {
    }


    System.out.println("Third stage.");

    //final map(only) job. relabeling

    conf = new JobConf(DiscFinder.class);
    conf.setJobName("final");

    conf.setOutputKeyClass(IntWritable.class);
    conf.setOutputValueClass(Text.class);

    conf.setMapperClass(FinalMap.class);
    conf.setNumReduceTasks(0); //no reduce
    conf.setNumMapTasks(NumSlot);

    conf.setCombinerClass(FinalReduce.class);
    conf.setReducerClass(FinalReduce.class);

    conf.setInputFormat(TextInputFormat.class);
    conf.setOutputFormat(TextOutputFormat.class);

    conf.setStrings("NFSPath", NFSPath);
    conf.setStrings("LocalPath", LocalPath);
    conf.setStrings("TempPath", TempPath);
    conf.setStrings("InputPath", InputPath);
    conf.setStrings("OutputPath", OutputPath);

    //input: a directory containing a file with literally number 0 - (NumPartitions-1) (one for each line)
    //output: whatever it is
    FileInputFormat.setInputPaths(conf, new Path[] { new Path("trivial") });
    FileOutputFormat.setOutputPath(conf, new Path(OutputPath));

    JobClient.runJob(conf);
 }

  public static class SampleCombine extends MapReduceBase implements Reducer <Text, IntWritable, Text, IntWritable> {
    public void reduce (Text key, Iterator<IntWritable> values, OutputCollector<Text, IntWritable> output, Reporter reporter) throws IOException{
      while (values.hasNext())
        output.collect(key, values.next());
    }
  }

  //sample
  public static class SampleReduce extends MapReduceBase implements Reducer <Text, IntWritable, Text, IntWritable> {

    public void reduce (Text key, Iterator<IntWritable> values, OutputCollector<Text, IntWritable> output, Reporter reporter) throws IOException {
      int sum = 0;
      while (values.hasNext()) {
        sum+=values.next().get();
      }
      output.collect(key, new IntWritable(sum));
    }
  }

  public static class SampleMap extends MapReduceBase
    implements Mapper<LongWritable, Text, Text, IntWritable> {

    private final static IntWritable one = new IntWritable(1);
    private Text word = new Text();

    public void map(LongWritable key, Text value, OutputCollector<Text, IntWritable> output, Reporter reporter) throws IOException {
      String line = value.toString();
      StringTokenizer itr = new StringTokenizer(line);
      int n=SampleRate; //intuitive sample. SampleRate is a prime. select 1/SampleRate point
      long temp = key.get();
      int tempvalue = (int)temp;
      if (n != 0)
        if (tempvalue % n == (n-1))
        {
          itr.nextToken();
          String a = (itr.nextToken()+" "+itr.nextToken()+" "+itr.nextToken());
          word.set(a);
          output.collect(word,one);
        }
    }
  }

  public static class FinalReduce extends MapReduceBase
    implements Reducer<LongWritable, Text, LongWritable, Text>
  {
    public void reduce(LongWritable paramLongWritable, Iterator<Text> paramIterator, OutputCollector<LongWritable, Text> paramOutputCollector, Reporter paramReporter)
      throws IOException
    {
    }
  }

  public static class FinalMap extends MapReduceBase
    implements Mapper<LongWritable, Text, LongWritable, Text>
  {
    public static void deleteDirectory(String s)
    {
      File f = new File(s);
      File files[] = f.listFiles();

      for (int i=0;i<files.length;i++)
        files[i].delete();
    }

    static String NFSPath;
    static String LocalPath;
    static String TempPath;
    static String InputPath;
    static String OutputPath;

    public void configure(JobConf job) {
      NFSPath = job.getStrings("NFSPath", "")[0];
      LocalPath = job.getStrings("LocalPath", "")[0];
      InputPath = job.getStrings("InputPath", "")[0];
      TempPath = job.getStrings("TempPath", "")[0];
      OutputPath = job.getStrings("OutputPath", "")[0];
    }

    public void map(LongWritable paramLongWritable, Text paramText, OutputCollector<LongWritable, Text> paramOutputCollector, Reporter paramReporter)
      throws IOException
    {

      String str1;
      StringTokenizer stk;
      Object localObject2;
      Object localObject3;
      Object localObject4;
      Object localObject5;
      long l = System.currentTimeMillis();

      String str2 = paramText.toString();

      //first read the "transitional rules"
      BufferedReader localBufferedReader = new BufferedReader(new FileReader(NFSPath+"trans.trn"));

      //add the rules to a hashmap
      HashMap localHashMap = new HashMap();
      while ((str1 = localBufferedReader.readLine()) != null)
      {
        stk = new StringTokenizer(str1);

        localObject2 = stk.nextToken();
        localObject3 = stk.nextToken();
        localHashMap.put(localObject2, localObject3);
      }
      localBufferedReader.close();

      //first change the
      Object localObject1 = new File(NFSPath+"afofoutput/shell/" + str2 + ".grp");

      if (((File)localObject1).exists())
      {
        localObject2 = new BufferedWriter(new FileWriter(NFSPath+"afofoutput/shell/" + str2 + ".newgrp"));
        localBufferedReader = new BufferedReader(new FileReader(NFSPath+"afofoutput/shell/" + str2 + ".grp"));
        while ((str1 = localBufferedReader.readLine()) != null)
        {
          if (str1.length() < 5) {
            continue;
          }
          stk = new StringTokenizer(str1);

          localObject4 = stk.nextToken();
          localObject5 = stk.nextToken();

          if (localHashMap.get(localObject5) == null)
            ((BufferedWriter)localObject2).write(((String)localObject4) + " " + ((String)localObject5) + "\n");
          else
            ((BufferedWriter)localObject2).write(((String)localObject4) + " " + localHashMap.get(localObject5) + "\n");
        }
        ((BufferedWriter)localObject2).close();
        localBufferedReader.close();
      }

      for (int i = 0; i < NumPartitions; ++i)
      {
        try
        {
          localObject3 = new File(LocalPath+"AFOF/core/" + i + ".newgrp");
          if (!(((File)localObject3).exists()))
          {
            localObject3 = new File(LocalPath+"AFOF/core/" + i + ".grp");
            if (((File)localObject3).exists())
            {
              localBufferedReader = new BufferedReader(new FileReader(LocalPath+"AFOF/core/" + i + ".grp"));
              localObject4 = new BufferedWriter(new FileWriter(LocalPath+"AFOF/core/" + i + ".newgrp"));

              while ((str1 = localBufferedReader.readLine()) != null)
              {
                if (str1.length() < 5) {
                  continue;
                }
                localObject5 = new StringTokenizer(str1);

                String str3 = ((StringTokenizer)localObject5).nextToken();
                String str4 = ((StringTokenizer)localObject5).nextToken();

                if (localHashMap.get(str4) == null)
                  ((BufferedWriter)localObject4).write(str3 + " " + str4 + "\n");
                else
                  ((BufferedWriter)localObject4).write(str3 + " " + localHashMap.get(str4) + "\n");
              }
              ((BufferedWriter)localObject4).close();
              localBufferedReader.close();
            }
          }
        }
        catch (FileNotFoundException localFileNotFoundException)
        {
        }
      }

      //delete files..
      File f = new File(LocalPath);
      if (!f.exists())
        f.mkdirs();
    }
  }

  public static class Combine extends MapReduceBase
    implements Reducer<IntWritable, FOFpt, IntWritable, FOFpt>
  {
    public void reduce(IntWritable paramIntWritable, Iterator<FOFpt> paramIterator, OutputCollector<IntWritable, FOFpt> paramOutputCollector, Reporter paramReporter)
      throws IOException
    {
      while (paramIterator.hasNext())
        paramOutputCollector.collect(paramIntWritable, paramIterator.next());
    }
  }

  public static class Reduce extends MapReduceBase
    implements Reducer<IntWritable, FOFpt, IntWritable, Text>
  {
    static String NFSPath;
    static String LocalPath;
    static String TempPath;
    static String InputPath;
    static String OutputPath;

    public void configure(JobConf job) {
      NFSPath = job.getStrings("NFSPath", "")[0];
      LocalPath = job.getStrings("LocalPath", "")[0];
      InputPath = job.getStrings("InputPath", "")[0];
      TempPath = job.getStrings("TempPath", "")[0];
      OutputPath = job.getStrings("OutputPath", "")[0];
    }

    public void deleteDirectory(String s)
    {
      File file = new File(s);
      File[] files = file.listFiles();
      for (int i=0;i<files.length;i++)
        files[i].delete();
    }

    public void reduce(IntWritable paramIntWritable, Iterator<FOFpt> paramIterator, OutputCollector<IntWritable, Text> paramOutputCollector, Reporter paramReporter)
      throws IOException
    {
      Date d1 = new Date();
      long l1 = System.currentTimeMillis();

      String str = LocalPath;
      File f = new File(str);

      if (!f.exists())
        f.mkdirs();

      //first prepare the input file for fof
      BufferedOutputStream localBufferedOutputStream = new BufferedOutputStream(new FileOutputStream(str + paramIntWritable));

      byte[] arrayOfByte = new byte[4];

      int i = 0;
      arrayOfByte = DiscFinder.intToByteArray(i);
      localBufferedOutputStream.write(arrayOfByte);
      localBufferedOutputStream.write(arrayOfByte);

      int j = 66;
      arrayOfByte = DiscFinder.intToByteArray(j);
      localBufferedOutputStream.write(arrayOfByte);

      int k = 3;
      arrayOfByte = DiscFinder.intToByteArray(k);
      localBufferedOutputStream.write(arrayOfByte);

      int l = -1; //length
      arrayOfByte = DiscFinder.intToByteArray(l);
      localBufferedOutputStream.write(arrayOfByte);

      int i1 = 0;
      arrayOfByte = DiscFinder.intToByteArray(i1);
      localBufferedOutputStream.write(arrayOfByte);

      int i2 = 0;
      arrayOfByte = DiscFinder.intToByteArray(i2);
      localBufferedOutputStream.write(arrayOfByte);

      float f1 = 0.0F;

      int totalNum=0;

      while (paramIterator.hasNext())
      {
        FOFpt pt = (FOFpt)paramIterator.next();

        arrayOfByte = DiscFinder.floatToByteArray(f1);
        for (int i4 = 0; i4 < 2; ++i4) {
          localBufferedOutputStream.write(arrayOfByte);
        }
        int i4 = 0;
        int i5 = 0;

        if (pt.inShell==1)
        {
          i4 = 1;
        }

        i5 = (int)(pt.id);

        float f2 = pt.px;
        arrayOfByte = DiscFinder.floatToByteArray(f2);
        localBufferedOutputStream.write(arrayOfByte);
        f2 = pt.py;
        arrayOfByte = DiscFinder.floatToByteArray(f2);
        localBufferedOutputStream.write(arrayOfByte);
        f2 = pt.pz;
        arrayOfByte = DiscFinder.floatToByteArray(f2);
        localBufferedOutputStream.write(arrayOfByte);

        arrayOfByte = DiscFinder.intToByteArray(i4);
        localBufferedOutputStream.write(arrayOfByte);
        arrayOfByte = DiscFinder.intToByteArray(i5);
        localBufferedOutputStream.write(arrayOfByte);

        arrayOfByte = DiscFinder.floatToByteArray(f1);
        for (int i6 = 0; i6 < 5; ++i6) {
          localBufferedOutputStream.write(arrayOfByte);
        }

        totalNum++;
      }

      localBufferedOutputStream.flush();
      localBufferedOutputStream.close();

      //trick!!
      RandomAccessFile raf = new RandomAccessFile(str + paramIntWritable,"rw");
      raf.seek(16);

      arrayOfByte = DiscFinder.intToByteArray(totalNum);
      raf.write(arrayOfByte);
      raf.close();


      long l2 = System.currentTimeMillis();

      //executing fof
      Runtime localRuntime = Runtime.getRuntime();

      String localObject[] = { "/bin/sh", "-c",
                              NFSPath+"fof -e " + Epsilon + " -g -m 1 -o "+LocalPath+"AFOF/core/"
                              + paramIntWritable.get() + " -b " + (MaxPointEF * paramIntWritable.get())
                              + " -s "+NFSPath+"afofoutput/shell/"+paramIntWritable.get()
                              + " < "+LocalPath+"AFOF/afofinput/" + paramIntWritable.get() };
      Process localProcess = localRuntime.exec(localObject);

      InputStream localInputStream = localProcess.getInputStream();
      InputStreamReader localInputStreamReader = new InputStreamReader(localInputStream);
      BufferedReader localBufferedReader = new BufferedReader(localInputStreamReader);
      String str2 = null;
      System.out.println("<output></output>");
      while ((str2 = localBufferedReader.readLine()) != null);
      try
      {
        if (localProcess.waitFor() != 0)
          System.err.println("exit value = " + localProcess.exitValue());
      }
      catch (Exception localException)
      {
      }
      localRuntime = null;
      localProcess = null;
      System.gc();
      System.runFinalization();

    }
  }

  //new map
  //use O(log #cubes) rather than O(cube)
  //input: bdr, created from cc.class
  public static class Map extends MapReduceBase
    implements Mapper<LongWritable, Text, IntWritable, FOFpt>
  {
    private static final IntWritable one = new IntWritable(1);
    private Text word;
    private static double[][] arrayOfDouble1 = null;
    private ArrayList localArrayList;
    private Boolean inS = false;
    private static float dis = Epsilon;

    static String NFSPath;
    static String LocalPath;
    static String TempPath;
    static String InputPath;
    static String OutputPath;

    public void configure(JobConf job) {
      NFSPath = job.getStrings("NFSPath", "")[0];
      LocalPath = job.getStrings("LocalPath", "")[0];
      InputPath = job.getStrings("InputPath", "")[0];
      TempPath = job.getStrings("TempPath", "")[0];
      OutputPath = job.getStrings("OutputPath", "")[0];
    }

    public Map()
    {
      this.word = new Text();
      if (arrayOfDouble1 == null)
      {
        arrayOfDouble1 = new double[10][];
        int [] counter = new int[10];
        for (int i=0;i<10;i++)
        {
          counter[i]=0;
          arrayOfDouble1[i] = new double[(int)Math.floor(Math.pow(2,i)+0.01)];
        }
        try {
          BufferedReader br = new BufferedReader(new FileReader(NFSPath+"bdr"));

          String tempS;
          for (int n=0;n<NumPartitions-1;n++)
          {
            tempS = br.readLine();
            StringTokenizer itr = new StringTokenizer(tempS);
            int level = Integer.parseInt(itr.nextToken())-1;
            double bd = Double.parseDouble(itr.nextToken());
            arrayOfDouble1[level][counter[level]] = bd;
            counter[level]++;
          }
        } catch(Exception e){}
      }
    }

    public void returnGroupIndex(float f1, float f2, float f3, int level,int index)
    {
      if (level >= 10)
      {
        localArrayList.add(index);
        return;
      }
      else
      {
        if (level % 3 ==0)
        {
          if (Math.abs(f1 - arrayOfDouble1[level][index])<dis / 2.0F)
          {
            inS = true;
            returnGroupIndex(f1,f2,f3,level+1,index*2);
            returnGroupIndex(f1,f2,f3,level+1,index*2+1);
          }
          else if (f1 < arrayOfDouble1[level][index])
            returnGroupIndex(f1,f2,f3,level+1,index*2);
          else
            returnGroupIndex(f1,f2,f3,level+1,index*2+1);
        }
        else if (level % 3 ==1)
        {
          if (Math.abs(f2 - arrayOfDouble1[level][index])<dis / 2.0F)
          {
            inS = true;
            returnGroupIndex(f1,f2,f3,level+1,index*2);
            returnGroupIndex(f1,f2,f3,level+1,index*2+1);
          }
          else if (f2 < arrayOfDouble1[level][index])
            returnGroupIndex(f1,f2,f3,level+1,index*2);
          else
            returnGroupIndex(f1,f2,f3,level+1,index*2+1);
        }
        else
        {
          if (Math.abs(f3 - arrayOfDouble1[level][index])<dis / 2.0F)
          {
            inS = true;
            returnGroupIndex(f1,f2,f3,level+1,index*2);
            returnGroupIndex(f1,f2,f3,level+1,index*2+1);
          }
          else if (f3 < arrayOfDouble1[level][index])
            returnGroupIndex(f1,f2,f3,level+1,index*2);
          else
            returnGroupIndex(f1,f2,f3,level+1,index*2+1);
        }
      }
    }

    public void map(LongWritable paramLongWritable, Text paramText, OutputCollector<IntWritable, FOFpt> paramOutputCollector, Reporter paramReporter) throws IOException {

      inS = false;
      localArrayList =new ArrayList();
      String str = paramText.toString();

      int i = 0;
      DiscFinder.FOFpt localFOFpt = new DiscFinder.FOFpt();
      float[] arrayOfFloat = new float[3];

      StringTokenizer localStringTokenizer = new StringTokenizer(str);

      long id = Long.parseLong(localStringTokenizer.nextToken());
      for (int k = 0; k < 3; ++k)
      {
        arrayOfFloat[k] = Float.parseFloat(localStringTokenizer.nextToken());
      }

      localFOFpt.px = arrayOfFloat[0];
      localFOFpt.py = arrayOfFloat[1];
      localFOFpt.pz = arrayOfFloat[2];
      localFOFpt.id = id;

      returnGroupIndex(localFOFpt.px, localFOFpt.py, localFOFpt.pz, 0, 0);

      for (int l = 0; l < localArrayList.size(); ++l)
      {
        if (inS==true)
          localFOFpt.inShell=1;
        else
          localFOFpt.inShell=0;

        int grpIndex = ((Integer)localArrayList.get(l)).intValue();
        paramOutputCollector.collect(new IntWritable(grpIndex), localFOFpt);
      }
    }
  }

  public static class FOFpt
    implements Writable
  {
    public int inShell;
    public float px;
    public float py;
    public float pz;
    public long id;

    public void write(DataOutput paramDataOutput)
      throws IOException
    {
      paramDataOutput.writeFloat(this.px);
      paramDataOutput.writeFloat(this.py);
      paramDataOutput.writeFloat(this.pz);
      paramDataOutput.writeInt(this.inShell);
      paramDataOutput.writeLong(this.id);
    }

    public void readFields(DataInput paramDataInput) throws IOException {
      this.px = paramDataInput.readFloat();
      this.py = paramDataInput.readFloat();
      this.pz = paramDataInput.readFloat();
      this.inShell = paramDataInput.readInt();
      this.id = paramDataInput.readLong();
    }

    public void FOFpt(int paramInt, float paramFloat1, float paramFloat2, float paramFloat3, long id)
    {
      this.inShell = paramInt;
      this.px = paramFloat1;
      this.py = paramFloat2;
      this.pz = paramFloat3;
      this.id = id;
    }
  }

  public static class groupMember
  {
    Boolean inShell;
    int groupIndex;
  }
}
